	radix DEC
	LIST      P=18F252	; change also: Configure->SelectDevice from Mplab 
xtal EQU 20000000		; you may want to change: _XT_OSC_1H  _HS_OSC_1H  _HSPLL_OSC_1H
baud EQU 115200			; standard TinyBld baud rates: 115200 or 19200
	; The above 3 lines can be changed and buid a bootloader for the desired frequency and PIC type
	;#define ground 0		;0=final  1=offline
	
	;********************************************************************
	;	Tiny Bootloader		18F series		Size=100words
	;	claudiu.chiculita@ugal.ro
	;	http://www.ac.ugal.ro/staff/ckiku/software/picbootloader.htm
	;********************************************************************
	

	#include "../icdpictypes.inc"	;takes care of: #include "p18fxxx.inc",  max_flash, IdTypePIC

	;if ground==1
	;	#define prima_adresa max_flash-200
	;else
		#define prima_adresa max_flash-200		;100 words ocupa
	;endif


	;if ground==0
		__CONFIG	_CONFIG1H, _OSCS_OFF_1H & _HS_OSC_1H
		__CONFIG	_CONFIG2L, _BOR_ON_2L & _BORV_20_2L & _PWRT_ON_2L
		__CONFIG	_CONFIG2H, _WDT_OFF_2H & _WDTPS_128_2H
		__CONFIG	_CONFIG4L, _STVR_ON_4L & _LVP_OFF_4L & _DEBUG_OFF_4L
	;endif


RoundResult SET 0				; Rounding function.  RoundResult = Round(aa/bb)
Round	macro aa,bb
	LOCAL rr = aa/bb
	LOCAL d1 = aa - rr*bb
	LOCAL d2 = (rr+1)*bb - aa
RoundResult = rr
	if d1 >= d2
RoundResult++
	endif
	endm

AbsResult SET 0
Abs		macro nr
	if nr>=0
AbsResult = nr
	else
AbsResult = -nr
	endif
	endm
		
	Round xtal,(16*baud)
spbrg_value EQU RoundResult-1

	Round xtal,(16*(spbrg_value+1))
baud_real EQU RoundResult
	Abs(baud_real-baud)
errpercent EQU AbsResult*100/baud
	if errpercent > 4
		ERROR big error in baudrate: #v(errpercent)%
	endif
	if errpercent >= 2
		messg baudrate not exact: #v(errpercent)%
	endif




;----------------------------- PROGRAM ---------------------------------
	cblock 0
	crc
	i
	cnt1
	cnt2
	cnt3
	counter_hi
	counter_lo
	flag
	endc
	cblock 10
	buffer:64
	dummy4crc
	endc
	
SendL macro car
	movlw car
	movwf TXREG
	;rcall SendLWait
	endm

	
;0000000000000000000000000 RESET 00000000000000000000000000

		ORG     0x0000
		GOTO    IntrareBootloader
		

;&&&&&&&&&&&&&&&&&&&$$$$   START     &&&&&&&&&&&&&&&&&
;----------------------  Bootloader  ----------------------
;					 b6=1  EA ED 0? crc					eeprom
;	C1					U  H  L  nr  ...(nr)...  crc	flash
;	   id  ver  K
	
	ORG prima_adresa
	nop
	nop
	nop
	nop
	org prima_adresa+8
IntrareBootloader
			;init seriala
	movlw b'00100100'
	movwf TXSTA
	movlw spbrg_value
	movwf SPBRG
	movlw b'10010000'
	movwf RCSTA
					;wait for computer
	rcall Receive			
	sublw 0xC1			;Expect C1h
	bnz prima_adresa
	SendL IdTypePIC			;PIC type
MainLoop
	SendL 'K'
mainl
	clrf crc
	rcall Receive		;U
	movwf TBLPTRU
		movwf flag
	rcall Receive		;H
	movwf TBLPTRH
		movwf EEADR
	rcall Receive		;L
	movwf TBLPTRL
		movwf EEDATA

	rcall Receive		;count
	movwf i
	incf i
	lfsr FSR0, (buffer-1)
rcvoct						;read 64+1 bytes
		movwf TABLAT		; prepare for cfg; => store byte before crc
	rcall Receive
	movwf PREINC0
	decfsz i
	bra rcvoct
	
	tstfsz crc				;check crc
	bra ziieroare
		btfss flag,6			;convention to be 1 for eeprom
		bra noeeprom
		movlw b'00000100'		; Setup eeprom
		rcall Write
		bra waitwre
noeeprom
		btfss flag,7			;convention to be 1 for config
		bra noconfig
		tblwt*				;write TABLAT(byte before crc) to TBLPTR***
		movlw b'11000100'		; Setup cfg
		rcall Write
		bra waitwre
noconfig
				;write
eraseloop
	movlw	b'10010100'			; Setup erase
	rcall Write
	TBLRD*-					; point to adr-1
	
writebigloop	
	movlw 8					; 8groups
	movwf counter_hi
	lfsr FSR0,buffer
writesloop
	movlw 8					; 8bytes = 4instr
	movwf counter_lo
writebyte
	movf POSTINC0,w				; put 1 byte
	movwf TABLAT
	tblwt+*
	decfsz counter_lo
	bra writebyte
	
	movlw	b'10000100'			; Setup writes
	rcall Write
	decfsz counter_hi
	bra writesloop
waitwre	
	;btfsc EECON1,WR			;for eeprom writes (wait to finish write)
	;bra waitwre				; no need: round trip time with PC bigger than 4ms
	
	bcf EECON1,WREN				;disable writes
	bra MainLoop
	
ziieroare
	SendL 'N'
	bra mainl
	  
;******** procedures ******************

Write
	movwf EECON1
	movlw 0x55
	movwf EECON2
	movlw 0xAA
	movwf EECON2
	bsf EECON1,WR			;WRITE
	nop
	;nop
	return
	  
;SendLWait
;	btfss PIR1,TXIF		;empty?
;	bra SendLWait
;	movwf TXREG
;	return

Receive
	movlw xtal/2000000+1			; for 20MHz => 11 => 1second
	movwf cnt1
rpt2						
	clrf cnt2
rpt3
	clrf cnt3
rptc
		btfss PIR1,RCIF			;test RX
		bra notrcv
	    movf RCREG,w			;return in W
	    addwf crc,f				;compute crc
		return
notrcv
	decfsz cnt3
	bra rptc
	decfsz cnt2
	bra rpt3
	decfsz cnt1
	bra rpt2
				;timeout:
	bra prima_adresa
;*************************************************************
; After reset
; Do not expect the memory to be zero,
; Do not expect registers to be initialised like in catalog.



            END

